#ifndef DIGIT_H
#define DIGIT_H

#ifdef E
#undef E
#endif

#ifdef DIGIT
#define E extern
#else
#define E
#endif

void put_scr(signed int scr);
void put_kill_cosmo(signed int csm);
void put_line(signed int line);
void put_level(signed int level);

#endif
