/*****************************************************************
  £Ã£ï£ó£í£ï £Ç£á£î£ç £ô£è£å £Ð£õ£ú£ú£ì£å ver2.xx

            1996 Programed by  Eiju  Tsuboi s1021127@u-aizu.ac.jp
                 Grapghics by Takaaki Izumi s1021085@u-aizu.ac.jp
******************************************************************/

#define MAIN
#include <sys/times.h>
#include <signal.h>
#include "main.h"

void end_session()
{
  signal(SIGALRM,SIG_IGN);
  signal(SIGINT,SIG_IGN);
  x_end();
  exit(0);
}

void main(int argc,char **argv)
{
  ac=argc; av=argv;
  setup();
  
  for(;;)
  {
    if( (s_level=title()) == QUIT ) break;
    if( main_loop(s_level) == QUIT ) break;
    ranking();
  }

  end_session();
  exit(0);
}
