#ifndef TIMER_H
#define TIMER_H

#include <signal.h>
#include <sys/time.h>
#include <sys/times.h>
#include "setup.h"

#ifdef E
#undef E
#endif

#ifdef TIMER
#define E
#else
#define E extern
#endif

E struct itimerval itp;

/* Functions */
E void nap(int n);
E void init_timer();
#endif
