#ifndef MAIN_H
#define MAIN_H

#include <stdio.h>

#ifdef MAIN
#define E extern
#else
#define E
#endif

#define LEFT	0
#define RIGHT	1
#define DOWN	2

#define TITLE "COSMO GANG THE PUZZLE"
#define PRODUCE "1996   Program: Eiju Tsuboi   Graphics: Takaaki Izumi"
#define VERSION	"2.0.4"

#define SIZE 32		/* キャラクターサイズ */
#define HEIGHT 15	/* ゲーム画面の縦幅 */

/* エラーコード */
#define QUIT	-1
#define MEM_ERR	-2
#define X_ERR	-3
#define SYS_ERR -4

typedef struct {
  int score;
  int line;
  int cosmo;
  int level;
  int slevel;
  char name[9];
  int uid;
} UserStatus;

#ifdef USE_PAD
E int upad;
E char pad_dev[256];
#endif

/* 引数 */
E int ac;
E char **av;

E UserStatus user;	/* プレイヤーの各種データ */
E int s_level;		/* スタートレベル */

/* Functions */
E void end_session();
E void main(int argc,char **argv);

#endif
