/* 簡易ダンププログラム */

#include <stdio.h>
#include <sys/stat.h>

int get_fs( char *file_name )
{
  struct stat st;

  stat(file_name,&st);
  return st.st_size;
}

int main(int ac,char **av)
{
  FILE *fp;
  unsigned char c;
  int size;
  int i;

  if(ac>1)
  {
    if( (fp=fopen(av[1],"r"))==NULL )
    {
      printf("cannot open file %s\n",av[1]);
      exit(-1);
    }
  }
  else exit(-1);
  
  size=get_fs(av[1]);
  
  for(i=0;i<size;i++)
  {
    if(i%8==0) printf("  ");
    c=fgetc(fp);
    printf("0x%02x,",c);
    if((i+1)%8==0) printf("\n");
  }

  fclose(fp);

  exit(0);
}
