#ifndef SCORE_H
#define SCORE_H

#define put_num(WINATTR,C,X,Y)\
{\
  XCopyArea(disp,background,(WINATTR).pix,gc,\
	    (WINATTR).offset_x+1+X,scrwattr.offset_y+1+Y,16,16,X,Y);\
  place_chara((WINATTR).pix,C,X,Y);\
  XCopyArea(disp,(WINATTR).pix,(WINATTR).win,gc,X,0,16,16,X,Y);\
  XCopyArea(disp,background,(WINATTR).pix,gc,\
	    (WINATTR).offset_x+1+X,(WINATTR).offset_y+1+Y,16,16,X,Y);\
  place_chara((WINATTR).pix,C,X,Y);\
  XCopyArea(disp,(WINATTR).pix,(WINATTR).win,gc,X,Y,16,16,X,Y);\
}

#endif
